/***************************************************************************
 *   Copyright (C) 2005 by Michal Vaner                                    *
 *   vorner@centrum.cz                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VORLIB_TREE_H
#define VORLIB_TREE_H

#include "exception.h"
#include "stack.h"
#include <vector>

namespace vorlib {

class tree_e : public exception {
public:
  tree_e(int id = 0, const std::string &message = "");
};

/*
  ERROR IDs:
  -1: internal, newer should go out
  1: index not found
*/

template <class index_t, class data_t> class knot_c {
public:
  knot_c (const index_t &index, const data_t &data);//constructor
  knot_c (const knot_c &source);//copy constructor
  ~knot_c ();//destructor

  index_t index;//some items
  data_t data;
  knot_c *left, *right;//children

  void fill_indexes(std::vector<index_t> &indexes) const;
  void fill_data(std::vector<data_t> &data) const;
};

template <class index_t, class data_t> class search_tree {
protected:

  knot_c<index_t, data_t> *root;//the top-level item

private:

  void remove(knot_c<index_t, data_t> *item, knot_c<index_t, data_t> *above, bool left);
  bool check(knot_c<index_t, data_t> *item) const;

public:
  search_tree();//constructor
  search_tree(const search_tree &source);
  virtual ~search_tree();//destructor

  data_t &get(const index_t &index);//get the item
  const data_t &get(const index_t &index) const;//can change like data_t &data = tree.get; data = ...
  data_t &operator[](const index_t &index);//it works as a l-value, like tree[index]=data
  const data_t &operator[](const index_t &index) const;
  data_t *try_get(const index_t &index);//get a pointer to that data. It is not a good idea to free it
  const data_t *try_get(const index_t &index) const;//if not found, NULL

  void clear();//remove all

  bool contains(const index_t &index) const;
  bool operator%(const index_t &index) const;//is it there?
  search_tree &operator=(const search_tree &source);//copy it

  std::vector<index_t> get_indexes() const;
  std::vector<data_t> get_data() const;//return all, sorted by index

  bool reindex(const index_t &old_index, const index_t &new_index);//change index from old to new
  void remove(const index_t &index);//after this, it definitelly wont be inside, no mater weather it was before or not..
  virtual bool add(const index_t &index, const data_t &data);//put there ine item
  bool empty() const;//is it emty?
  bool check() const;//check, wether it is all OK, just for debuging the tree
};//for storing indexed data, index MUST be identical

template <class index_t, class data_t> class balanced_knot_c : public knot_c<index_t, data_t> {
private:
  typedef balanced_knot_c<index_t, data_t> *knot_p;
public:
  balanced_knot_c(const index_t &index, const data_t &data);//constructor
  balanced_knot_c(const balanced_knot_c *source);

  int depth() const;//get max depth of this and check the way in, weather it is OK

  short int sign;//how are we balanced ??
};

template <class index_t, class data_t> class balanced_search_tree : public search_tree<index_t, data_t> {
private:

  stack<balanced_knot_c<index_t, data_t> *> trace;//for back-tracing how we got there
  typedef balanced_knot_c<index_t, data_t> *knot_p;

  short signed balance(balanced_knot_c<index_t, data_t> *);//it is unbalanced, balance it
  void remove(balanced_knot_c<index_t, data_t> *knot);

public:
  balanced_search_tree();
  balanced_search_tree(const balanced_search_tree &source);
  balanced_search_tree &operator=(const balanced_search_tree &source);
  bool add(const index_t &index, const data_t &data);//add new item
  void remove(const index_t &index);//remove item
  bool depthtest() const;//check, weather it is balanced, for debuging this tree only
};//just a bit better version of the seartch_tree - it is faster with more gets, try_gets and contains than adds and removes

template <class index_t, class data_t> knot_c<index_t, data_t>::knot_c(const index_t &_index, const data_t &_data):index(_index), data(_data) {
  left = NULL;
  right = NULL;
}

template <class index_t, class data_t> knot_c<index_t, data_t>::knot_c(const knot_c &source) {
  index = source.index;
  data = source.data;
  if (source.left) left = new knot_c(*source.left); else left = NULL;//copy it's left if it exists
  if (source.right) right = new knot_c(*source.right); else right = NULL;
}

template <class index_t, class data_t> knot_c<index_t, data_t>::~knot_c() {
  if (left) delete left;
  if (right) delete right;
}

template <class index_t, class data_t> void knot_c<index_t, data_t>::fill_indexes(std::vector<index_t> &indexes) const {
  if (left) left -> fill_indexes(indexes);
  indexes.push_back(index);
  if (right) right -> fill_indexes(indexes);
}

template <class index_t, class data_t> void knot_c<index_t, data_t>::fill_data(std::vector<data_t> &data) const {
  if (left) left -> fill_data(data);
  data.push_back(this -> data);
  if (right) right -> fill_data(data);
}

template <class index_t, class data_t> search_tree<index_t, data_t>::search_tree() {
  root = NULL;
}

template <class index_t, class data_t> search_tree<index_t, data_t>::search_tree(const search_tree<index_t, data_t> &source) {
  if (source.root) root = new knot_c<index_t, data_t>(*source.root); else root = NULL;
}

template <class index_t, class data_t> search_tree<index_t, data_t> &search_tree<index_t, data_t>::operator=(const search_tree<index_t, data_t> &source) {
  clear();//kill all actually existing
  if (source.root) root = new knot_c<index_t, data_t>(*source.root);
  return *this;
}

template <class index_t, class data_t> search_tree<index_t, data_t>::~search_tree() {
  if (root) delete root;//remove them all
}

template <class index_t, class data_t> data_t &search_tree<index_t, data_t>::get(const index_t &index) {
  knot_c<index_t, data_t> *item = root;//start at the top
  while (item) {
    if (item -> index == index) return (item -> data);
    if (index < item -> index) item = item -> left;
    else item = item -> right;
  }
  throw new tree_e(1, "item not found");
}

template <class index_t, class data_t> const data_t &search_tree<index_t, data_t>::get(const index_t &index) const {
  knot_c<index_t, data_t> *item = root;//start at the top
  while (item) {
    if (item -> index == index) return (item -> data);
    if (index < item -> index) item = item -> left;
    else item = item -> right;
  }
  throw new tree_e(1, "item not found");
}

template <class index_t, class data_t> data_t &search_tree<index_t, data_t>::operator[](const index_t &index) {
  return (get(index));
}

template <class index_t, class data_t> const data_t &search_tree<index_t, data_t>::operator[](const index_t &index) const {
  return (get(index));
}

template <class index_t, class data_t> void search_tree<index_t, data_t>::clear() {
  if (root) delete root;
  root = NULL;//empty it
}

template <class index_t, class data_t> bool search_tree<index_t, data_t>::contains(const index_t &index) const {
  knot_c<index_t, data_t> *item = root;//start at the top one
  while (item) {
    if (item -> index == index) return (true);
    if (index < item -> index) item = item -> left;
    else item = item -> right;
  }
  return (false);//it's not there
}

template <class index_t, class data_t> bool search_tree<index_t, data_t>::operator%(const index_t &index) const {
  return (contains(index));
}

template <class index_t, class data_t> data_t *search_tree<index_t, data_t>::try_get(const index_t &index) {
  knot_c<index_t, data_t> *item = root;
  while (item) {
    if (item -> index == index) return &item -> data;//return pointer to it
    if (index < item -> index) item = item -> left;
    else item = item -> right;
  }
  return (NULL);
}

template <class index_t, class data_t> const data_t *search_tree<index_t, data_t>::try_get(const index_t &index) const {
  knot_c<index_t, data_t> *item = root;
  while (item) {
    if (item -> index == index) return &item -> data;//return pointer to it
    if (index < item -> index) item = item -> left;
    else item = item -> right;
  }
  return (NULL);
}

template <class index_t, class data_t> std::vector<index_t> search_tree<index_t, data_t>::get_indexes() const {
  std::vector<index_t> indexes;
  if (root) root -> fill_indexes(indexes);
  return indexes;
}

template <class index_t, class data_t> std::vector<data_t> search_tree<index_t, data_t>::get_data() const {
  std::vector<data_t> data;
  if (root) root -> fill_data(data);
  return data;
}

template <class index_t, class data_t> bool search_tree<index_t, data_t>::add(const index_t &index, const data_t &data) {
  knot_c<index_t, data_t> *new_item = new knot_c<index_t, data_t>(index, data);//create the new item
  if (!root) {
    root = new_item;
    return true;
  }
  knot_c<index_t, data_t> *item = root;//there is something definitelly
  for ( ; ; ) {//infinite loop
    if (index == item -> index) {//it is already there
      delete new_item;//it isn't used
      return (false);//it failed
    }
    if (index < item -> index) {//going left
      if (item -> left) item = item -> left;//move there
      else {//it's the place
        item -> left = new_item;
        return true;//so we placed it
      }
    } else {//going right
      if (item -> right) item = item -> right;//move right
      else {//empty place, it will be no longer empty...
        item -> right = new_item;
        return true;
      }
    }
  }
}

template <class index_t, class data_t> bool search_tree<index_t, data_t>::reindex(const index_t &old_index, const index_t &new_index) {
  data_t *data = try_get(old_index);//get it
  if (!data) return false;//this does not exist
  if (!add(new_index, *data)) return false;//it's there already
  remove(old_index);//remove the old one
  return true;//all ok
}

template <class index_t, class data_t> void search_tree<index_t, data_t>::remove(const index_t &index) {
  knot_c<index_t, data_t> *item = root;//get the top one
  knot_c<index_t, data_t> *above = NULL;//root has no parent
  bool left;
  while (item) {
    if (item -> index == index) {//remove it
      remove(item, above, left);//remove it
      return;//removed  .
    }
    above = item;//we are going down now
    if (index < item -> index) {//go to correct direction
      item = item -> left;
      left = true;
    } else {
      item = item -> right;
      left = false;
    }
  }
}

template <class index_t, class data_t> void search_tree<index_t, data_t>::remove(knot_c<index_t, data_t> *item, knot_c<index_t, data_t> *above, bool left) {
  if (!item -> left) {//we don't have left child, so we have max the right
    if (!above) root = item -> right;//we are the top one
    else if (left) above -> left = item -> right;//we are left child of the obove us
    else above -> right = item -> right;
    item -> right = NULL;//remove the pointer (we don;t want to free it as well
    delete item;//not needed
    return;
  }
  if (!item -> right) {
    if (!above) root = item -> left;//we definitelly have left child, we wouldn't continue in this func
    else if (left) above -> left = item -> left;
    else above -> right = item -> left;
    item -> left = NULL;
    delete item;
    return;
  }
  //so, we have both children
  knot_c<index_t, data_t> *lower = item -> right;//it does exist
  bool l_left = (lower -> left);//can we go left a little?
  knot_c<index_t, data_t> *l_above = item;//we start here
  while (lower -> left) {
    l_above = lower;
    lower = lower -> left;//go as far left as u can
  }
  //now we found something, replace us with it..
  item -> index = lower -> index;//move the lower item here (or just info from it :)) )
  item -> data = lower -> data;
  remove(lower, l_above, l_left);//remove the copyed item instead of me
                                 //note: it hase max 1 child (we coudn't go left)
}

template <class index_t, class data_t> bool search_tree<index_t, data_t>::empty() const {
  if (root) return false; else return true;//if it is emty, we don't have root
}

template <class index_t, class data_t> bool search_tree<index_t, data_t>::check() const {
  if (root) return check(root);
  return true;
}

template <class index_t, class data_t> bool search_tree<index_t, data_t>::check(knot_c<index_t, data_t> *item) const {
  if ((item -> left) && (item -> left -> index >= item -> index)) return false;//check, weather it is OK on left side
  if ((item -> right) && (item -> right -> index <= item -> index)) return false;//check, weather it is OK on right side
  if ((item -> left) && !check(item -> left)) return false;//check the left subtree
  if ((item -> right) && !check(item -> right)) return false;//check the right subtree
  return true;//no error found
}

template <class index_t, class data_t> balanced_knot_c<index_t, data_t>::balanced_knot_c(const index_t &index, const data_t &data) : knot_c<index_t, data_t>(index, data) {
  sign = 0;//we have no children, so we are absolutelly balanced
}

template <class index_t, class data_t> balanced_knot_c<index_t, data_t>::balanced_knot_c(const balanced_knot_c<index_t, data_t> *source) : knot_c<index_t, data_t>(source -> index, source -> data) {
  sign = source -> sign;
  if (source -> left) this -> left = new balanced_knot_c<index_t, data_t>((balanced_knot_c<index_t, data_t> *) source -> left); else this -> left = NULL;
  if (source -> right) this -> right = new balanced_knot_c<index_t, data_t>((balanced_knot_c<index_t, data_t> *) source -> right); else this -> right = NULL;
}

template <class index_t, class data_t> int balanced_knot_c<index_t, data_t>::depth() const {
  int leftdepth = (left) ? ((knot_p) left) -> depth() : 0;//take depth of left one
  int rightdepth = (right) ? ((knot_p) right) -> depth() : 0;//take depth of the right
  if (rightdepth - leftdepth != sign) throw new tree_e(-1, "bad sign");
  if ((sign < -1) || (sign > 1)) throw new tree_e(-1, "unbalanced");
  return (leftdepth > rightdepth) ? leftdepth + 1 : rightdepth + 1;//take the bigger of them and add one
}

template <class index_t, class data_t> balanced_search_tree<index_t, data_t>::balanced_search_tree() {
  search_tree<index_t, data_t>::root = NULL;
}

template <class index_t, class data_t> balanced_search_tree<index_t, data_t>::balanced_search_tree(const balanced_search_tree<index_t, data_t> &source):search_tree<index_t, data_t>(source) {
  if (source.root) search_tree<index_t, data_t>::root = new balanced_knot_c<index_t, data_t>((balanced_knot_c<index_t, data_t> *) source.root); else this -> root = NULL;//
}

template <class index_t, class data_t> balanced_search_tree<index_t, data_t> &balanced_search_tree<index_t, data_t>::operator=(const balanced_search_tree<index_t, data_t> &source) {
  this -> clear();
  if (source.root) this -> root = new balanced_knot_c<index_t, data_t>((balanced_knot_c<index_t, data_t> *) source.root); else this -> root = NULL;//
  return *this;
}

template <class index_t, class data_t> bool balanced_search_tree<index_t, data_t>::add(const index_t &index, const data_t &data) {
#define RET(result) {trace.clear(); return (result);}//it clears the trace and exits with result
  balanced_knot_c<index_t, data_t> *item = (knot_p) this -> root;//start searching
  trace.push(NULL);//root has no parent (it's orphant :))

  while (item) {
    if (index == item -> index) RET(false);//this index is already in use
    trace.push(item);
    if (index < item -> index) item = (knot_p) item -> left;
    else item = (knot_p) item -> right;
  }

  balanced_knot_c<index_t, data_t> *new_item = new balanced_knot_c<index_t, data_t>(index, data);//create the item
  item = trace.pop();//it's at last the one we put there before the while
  if (item) {
    if (index < item -> index) {
      item -> left = new_item;//so put it left
      item -> sign --;//it is getting left a little
    } else {
      item -> right = new_item;//same but to right
      item -> sign ++;
    }
  } else {
    this -> root = new_item;//OK, it's the first item. Nothing to do at all
  }

  while (item) {
    if (item -> sign == 0) RET(true);//it hasn't grown now, actually, it balanced by this adding
    bool grow;//do we grow above?
    if ((item -> sign == 1) || (item -> sign == -1)) grow = true;//it has grown by one, before there was 0
    else {
      int bal = balance(item);//balance it
      if (bal == 0) grow = true; else grow = false;//it is not grownig
    }
    if (!grow) RET(true)//not growing any more
    else {
      item = trace.pop();
      if (item) {
        if (index < item -> index) //we're on the left side
        item -> sign --;
        else item -> sign ++;
      }
    }
  }

  RET(true);
#undef RET//it is not needed
}

template <class index_t, class data_t> void balanced_search_tree<index_t, data_t>::remove(const index_t &index) {
#define EX {trace.clear(); return;}//emty the track end exit
  trace.push(NULL);//root has no parent
  balanced_knot_c<index_t, data_t> *item = (knot_p) this -> root;//start at the top

  while ((item) && (item -> index != index)) {//search until U find it
    trace.push(item);//save it as a parent of next one
    if (index < item -> index) item = (knot_p) item -> left;
    else item = (knot_p) item -> right;
  }

  if (!item) EX;

  remove(item);//so, U found it, than go and delete it

  EX;
#undef EX
}

template <class index_t, class data_t> short signed balanced_search_tree<index_t, data_t>::balance(balanced_knot_c<index_t, data_t> *item) {
  if (item -> sign == 2) {//it is hevier at the right side (so the right side exists for sure)
    /*the situation looks like this
         Item (-2)
         /   \
        X1   Right (?)
            /    \
           X2    X3

    */
    balanced_knot_c<index_t, data_t> *right = (knot_p) item -> right;//get the right one
    if (right -> sign >= 0) {//now just single rotation
      /*
        We want it like this    But in vars, it will be, I will swap the data inside
            Right                         Item
           /    \                        /    \
        Item    X3                     Right  X3
        /  \                           /  \
       X1  X2                         X1  X2
      */
      balanced_knot_c<index_t, data_t> *tmp = (knot_p) item -> left;//place somewhere X1
      item -> left = right;//and put there the Right. That empties item -> right
      item -> right = right -> right;//so place there X2
      right -> right = right -> left;//move X1 to right of right
      right -> left = tmp;//and return X1 to them
      //now, swap the data in right and item, so it is placed correctly
      data_t dtmp = item -> data;
      item -> data = right -> data;
      right -> data = dtmp;//swap data
      index_t itmp = item -> index;
      item -> index = right -> index;
      right -> index = itmp;//and swap indexes
      //now, how is it balanced?
      if (right -> sign == 0) {
        right -> sign = 1;
        item -> sign = -1;//so, we splitted the sign into two
        return 0;//and the height didn;t change
      } else {
        right -> sign = 0;
        item -> sign = 0;//it balanced itself
        return -1;//and it lowered
      }
    } else {
      /* Now the problem is worse:
             Item (2)
            /    \
           X1   Right (-) --> it has left son
               /      \
            Lower(?)   X4
            /   \
           X2   X3
        What I want to make of it is this:
              Lower (0)
             /     \
          Item(?)  Right(?)
          /  \      /  \
         X1  X2    X3  X4
        But in memory it will be like this (with some swaping of the insides)
               Item
              /    \
            Lower  Right
            / \     / \
           X1 X2   X3 X4
      */
      balanced_knot_c<index_t, data_t> *lower = (knot_p) right -> left;//so get right's left son
      balanced_knot_c<index_t, data_t> *tmp = (knot_p) item -> left;//put X1 somewhere
      item -> left = lower;//put lower where it belongs
      right -> left = lower -> right;//put X3 under Right
      lower -> right = lower -> left;//move X2 left under Lower(witch will be item later...)
      lower -> left = tmp;//put X1 under lower
      //so, they are connected like they should, now some insides to change - swap Item and Lower
      data_t dtmp = item -> data;
      item -> data = lower -> data;
      lower -> data = dtmp;//swap data
      index_t itmp = item -> index;
      item -> index = lower -> index;
      lower -> index = itmp;//adn swap index
      //how about how is it balanced now?
      //There's one place we don't know how it was balanced - the lower (we didn't change the sign yet, so it's still there, not in item)
      switch (lower -> sign) {
        case 1: lower -> sign = -1;
                right -> sign = 0;
                break;
        case 0: lower -> sign = 0;
                right -> sign = 0;
                break;
        case -1:lower -> sign = 0;
                right -> sign = 1;
                break;
      }
      item -> sign = 0;//it is balanced for sure
      return -1;//it definitelly lowered by one
    }
  } else {//then it should be -2 - it's unbalanced the other side
    /*situation:
       Item (-2)
      /   \
    Left  X3
   /  \
  X1  X2
    */
    balanced_knot_c<index_t, data_t> *left = (knot_p) item -> left;
    if (left -> sign <= 0) {//single rotation
      /*
      I want this:             However, it will be other way stored in vars, I'll Xchange the insides:
        Left                           Item
       /   \                           /   \
      X1   Item                       X1  Left
           /  \                            / \
          X2  X3                          X2 X3
      */
      balanced_knot_c<index_t, data_t> *tmp = (knot_p) item -> right;//copy there X3
      item -> right = left;//move it left
      item -> left = left -> left;//move X1 there
      left -> left = left -> right;//put it more left
      left -> right = tmp;//and place X3 there
      //now changing the insides of Left and Item
      data_t dtmp = left -> data;
      left -> data = item -> data;
      item -> data = dtmp;//data
      index_t itmp = left -> index;
      left -> index = item -> index;
      item -> index = itmp;//swap indexes
      //adn now, how is it balanced?
      if (left ->sign < 0) {
        left -> sign = 0;
        item -> sign = 0;
        return -1;
      } else {
        item -> sign = 1;
        left -> sign = -1;
        return 0;
      }
    } else {//double rotation
      /*actual situation:
            Item(-2)
           /      \
         Left(+)  X4
         /  \
        X1  Lower(?)
            /   \
           X2   X3
        I want:
              Lower(0)
             /     \
           Left(?) Item(?)
           /  \     /  \
          X1  X2   X3  X4
        But in memory it will be like this:
              Item
             /    \
            Left  Lower
           /  \    / \
          X1  X2  X3 X4
      */
      balanced_knot_c<index_t, data_t> *lower = (knot_p) left -> right;//it's left's right son
      balanced_knot_c<index_t, data_t> *tmp = (knot_p) item -> right;//save X4 there
      item -> right = lower;//move in that place Lower (it will be Item one time)
      left -> right = lower -> left;//place X2 under left
      lower -> left = lower -> right;//X3 move from right to left under lower
      lower -> right = tmp;//and X4 get's home as a right son of lower
      //now swap insides of Lower and Item
      data_t dtmp = item -> data;
      item -> data = lower -> data;
      lower -> data = dtmp;//there went data
      index_t itmp = item -> index;
      item -> index = lower -> index;
      lower -> index = itmp;//and index
      //now, what signs to put there
      switch (lower -> sign) {
        case 1:
          left -> sign = -1;
          lower -> sign = 0;
          break;
        case 0:
          left -> sign = 0;
          lower -> sign = 0;
          break;
        case -1:
          left -> sign = 0;
          lower -> sign = 1;
          break;
      }
      item -> sign = 0;
      return -1;
    }
  }
  return 1;//means error, this sould never happen
}

template <class index_t, class data_t> void balanced_search_tree<index_t, data_t>::remove(balanced_knot_c<index_t, data_t> *knot) {
  bool cont;//do we continue lowering the branch?
  bool left;//was it left child?
  balanced_knot_c<index_t, data_t> *parent = trace.pop();//get it's parent
  if (!knot -> left) {//it has even no children or only the right
    cont = true;
    if (parent) {
      if (parent -> left == knot) {
        parent -> left = knot -> right;
        left = true;
      } else {
        parent -> right = knot -> right;
        left = false;
      }
    } else {
      this -> root = knot -> right;//so, it was the top-one, place the only child(if it is) there
      cont = false;//nowhere to continue
    }
    knot -> right = NULL;//don't free it's children
    delete knot;//remove the knot
  } else if (!knot -> right) {//it has only left child
    cont = true;
    if (parent) {
      if (parent -> left == knot) {
        parent -> left = knot -> left;
        left = true;
      } else {
        parent -> right = knot -> left;
        left = false;
      }
    } else {
      this -> root = knot -> left;
      cont = false;
    }
    knot -> left = NULL;
    delete knot;
  } else {//it has both children
    cont = false;//we do not delete here
    trace.push(parent);
    trace.push(knot);//this one is on the way as well
    balanced_knot_c<index_t, data_t> *lower = (knot_p) knot -> left;//go first one left
    while (lower -> right) {//and right as much as posible
      trace.push(lower);
      lower = (knot_p) lower -> right;
    }
    knot -> index = lower -> index;
    knot -> data = lower -> data;
    remove(lower);
  }

  while (cont) {
    knot = parent;//move one up
    parent = trace.pop();
    if (left) knot -> sign ++;//balancing a little right
    else knot -> sign --;
    if (knot -> sign == 0) cont = true;//it got balanced, but lowered
    else if ((knot -> sign == 2) || (knot -> sign == -2)) {
      cont = (balance(knot) == -1);//it lowered as well
    }
    else cont = false;//this level is not afected any more
    if (!parent) break;//I have no parent, nowhere to go
    left = (parent -> left == knot);
  }
}

template <class index_t, class data_t> bool balanced_search_tree<index_t, data_t>::depthtest() const {
  if (!root) return true;
  try {
    ((knot_p) root) -> depth();//run trough it and check depths
    return true;
  }
  catch (tree_e *e) {
    if (e -> id == -1) return false;//it is unbalanced
    throw e;//it is something else
  }
}

}

#endif



